using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Events;

namespace SS
{
  public class SmartSpawnerBehaviour : MonoBehaviour
  {
    public string ItemName = "Brick";
    public float LineHeight = 1.5f;
    public float ItemScale = 0f;
    public bool SnapAngle = false;
    public bool FreezeItem = false;
    public bool StartUse = false;
    public float ItemCharge = 0f;
    public float ItemForce = 0f;
	public float ItemTemperature = 0f;
    public GameObject SpawnerLineObj;
    public LineRenderer SpawnerLine;
    public bool ParticleMode = false;
    public bool AdvancedMode = false;
    public Color s_color = Color.green;

    public GameObject SpawnerGlow;
    public SpriteRenderer SpawnerGlowSprite;

    private void Start()
    {
      SpawnerLineObj.transform.eulerAngles = transform.eulerAngles;
	  SpawnerLineObj.transform.localScale = transform.localScale;
      SpawnerGlow.transform.eulerAngles = transform.eulerAngles;
	  SpawnerGlow.transform.localScale = transform.localScale;
      SpawnerLine.SetPositions(new[] {Vector3.zero, Vector3.up * LineHeight});
      SetSpawnerColor(s_color);
    }
	
    private void Awake()
    {
      string[] particles = {"BlasterImpact", "FuseBlown", "MetalHit", "Disintegration", "BlasterImpactHole", "Spark", "WoodHit", "PinkExplosion", "EnormousExplosion", "BloodExplosion", "BigZap", "BigExplosion", "HugeZap", "Ricochet", "BrokenElectronicsSpark", "Explosion", "HugeZap", "RedBarrelExplosion", "Flash", "IonExplosion", "Vapor"};

      GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(
      new ContextMenuButton("startButton", "-----------", "-----------", () =>
      {
      }));
      GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(
      new ContextMenuButton("itemButton", "Set item", "Set item", () =>
      {
        DialogBox dialog = (DialogBox)null;
        dialog = DialogBoxManager.TextEntry("Enter the name of the item or particle\n<color=green><size=28>Selected: "+ItemName, "Text", new DialogButton("Apply", true, new UnityAction[1]
        {
          (UnityAction)(() => {
          if(dialog.EnteredText != "")
          {
            if(!AdvancedMode)
            {
              foreach(string x in particles)
              {
                if (dialog.EnteredText.Contains(x))
                {
                  ParticleMode = true;
                  break;
                }
                else
                {
                  ParticleMode = false;
                }
              }
              if(ParticleMode)
              {
                ItemName = dialog.EnteredText;
                Notify("Selected particle to spawn: <color=yellow>"+dialog.EnteredText+"</color>");
              }
              else
              {
                if(dialog.EnteredText != "Zombie")
                {
                  SpawnableAsset FindAsset = ModAPI.FindSpawnable(dialog.EnteredText);
                  if(FindAsset)
                  {
                    ItemName = dialog.EnteredText;
                    Notify("Selected item to spawn: <color=yellow>"+dialog.EnteredText+"</color>");
                  }
                  else
                  {
                    Notify("Item <color=yellow><"+dialog.EnteredText+"></color> not found!");
                  }
                }
                else
                {
                  ItemName = dialog.EnteredText;
                  Notify("Selected item to spawn: <color=yellow>"+dialog.EnteredText+"</color>");
                }
              }
            }
            else
            {
              UnityEngine.Object[] prefabs = Resources.FindObjectsOfTypeAll(typeof(GameObject));
              foreach(UnityEngine.Object prefab in prefabs)
              {
                if(prefab.name == dialog.EnteredText)
                {
                  ItemName = dialog.EnteredText;
                  Notify("Selected item to spawn: <color=yellow>"+dialog.EnteredText+"</color>");
                  break;
                }
              }
            }
          }
          else
          {
            Notify("<color=red>You have not entered the text!</color>");
          }
          })
        }),
		new DialogButton("Random", true, (UnityAction)(() =>
		{
			UnityEngine.Object[] assets = Resources.FindObjectsOfTypeAll(typeof(SpawnableAsset));
			UnityEngine.Object asset = assets[Random.Range(0, assets.Length)];
			ItemName = asset.name;
            Notify("Selected item to spawn: <color=yellow>"+asset.name+"</color>");
		})),
        new DialogButton("Cancel", true, (UnityAction)(() => dialog.Close())));
      }));
	  
      GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("heightButton", "Set height", "Set height", () =>
      {
        DialogBox dialog = (DialogBox)null;
        dialog = DialogBoxManager.TextEntry("Enter the height of the spawn point.\n<color=green><size=28>Selected: "+LineHeight, "Number", new DialogButton("Apply", true, new UnityAction[1]
        {
          (UnityAction)(() => {
          float setHeight;
          if(float.TryParse(dialog.EnteredText, out setHeight))
          {
            if(setHeight > 0)
            {
              LineHeight = setHeight;
              SpawnerLine.SetPositions(new[] {Vector3.zero, Vector3.up * LineHeight});
            }
            else
            {
              Notify("<color=red>Number is too small!</color>");
            }
          }
          else
          {
            Notify("<color=red>You have not entered the number!</color>");
          }
          })
        }),
        new DialogButton("Cancel", true, (UnityAction)(() => dialog.Close())));
      }));
	  
      GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("spawnerColorButton", "Spawner color", "Spawner color", () =>
      {
        DialogBox dialog = (DialogBox)null;
        dialog = DialogBoxManager.TextEntry("Set spawner color\n<red, green, blue, alpha(1/0)>\n<color=green>Example: 0, 1, 0, 1</color>", "red, green, blue, alpha(1/0)", new DialogButton("Apply", true, new UnityAction[1]
        {
          (UnityAction)(() => {
            string[] colorText = dialog.EnteredText.Split(", "[0]);
            Color c = new Color();
            if(colorText.Length == 4)
            {
              c.r = float.Parse(colorText[0]);
              c.g = float.Parse(colorText[1]);
              c.b = float.Parse(colorText[2]);
              c.a = float.Parse(colorText[3]);
              s_color = c;
              SetSpawnerColor(s_color);
              Notify("Spawner color changed to: <color=yellow>"+dialog.EnteredText+"</color>");
            }
            else
            {
              Notify("<color=red>Error.</color> (Example: 0, 1, 0, 1)");
            }
          })
        }),
        new DialogButton("Cancel", true, (UnityAction)(() => dialog.Close())));
      }));
	  
      GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("itemScaleButton", "Item scale", "Item scale", () =>
      {
        DialogBox dialog = (DialogBox)null;
        dialog = DialogBoxManager.TextEntry("Enter the scale of the spawned item.\n<color=green><size=28>Selected: "+ItemScale, "Number", new DialogButton("Apply", true, new UnityAction[1]
        {
          (UnityAction)(() => {
          float setScale;
          if(float.TryParse(dialog.EnteredText, out setScale))
          {
            ItemScale = setScale;
            if(setScale != 0)
            {
              Notify("The item will be spawned with a scale: <color=yellow>"+setScale+"</color>");
            }
            else
            {
              Notify("The spawned object will not be scaled.");
            }
          }
          else
          {
            Notify("<color=red>You have not entered the number!</color>");
          }
          })
        }),
        new DialogButton("Cancel", true, (UnityAction)(() => dialog.Close())));
      }));
	  
      GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("itemTemperatureButton", "Item temperature", "Item temperature", () =>
      {
        DialogBox dialog = (DialogBox)null;
        dialog = DialogBoxManager.TextEntry("Enter the temperature of the spawned item.\n<color=green><size=28>Selected: "+ItemTemperature, "Number", new DialogButton("Apply", true, new UnityAction[1]
        {
          (UnityAction)(() => {
          float setTemperature;
          if(float.TryParse(dialog.EnteredText, out setTemperature))
          {
            ItemTemperature = setTemperature;
			Notify("The item will be with temperature: <color=yellow>"+setTemperature+"</color>");
          }
          else
          {
            Notify("<color=red>You have not entered the number!</color>");
          }
          })
        }),
        new DialogButton("Cancel", true, (UnityAction)(() => dialog.Close())));
      }));
	  
      GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("itemChargeButton", "Item charge", "Item charge", () =>
      {
        DialogBox dialog = (DialogBox)null;
        dialog = DialogBoxManager.TextEntry("Enter the charge strength of the spawned item.\n<color=green><size=28>Selected: "+ItemCharge, "Number", new DialogButton("Apply", true, new UnityAction[1]
        {
          (UnityAction)(() => {
          float setCharge;
          if(float.TryParse(dialog.EnteredText, out setCharge))
          {
            ItemCharge = setCharge;
            if(setCharge != 0)
            {
              Notify("The item will be pushed out with force: <color=yellow>"+setCharge+"</color>");
            }
            else
            {
              Notify("Item will not be pushed out.");
            }
          }
          else
          {
            Notify("<color=red>You have not entered the number!</color>");
          }
          })
        }),
        new DialogButton("Cancel", true, (UnityAction)(() => dialog.Close())));
      }));
	  
      GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("itemPushButton", "Pushing force", "Pushing force", () =>
      {
        DialogBox dialog = (DialogBox)null;
        dialog = DialogBoxManager.TextEntry("Enter the pushing force of the spawned object.\n<color=green><size=28>Selected: "+ItemForce, "Number", new DialogButton("Apply", true, new UnityAction[1]
        {
          (UnityAction)(() => {
          float setForce;
          if(float.TryParse(dialog.EnteredText, out setForce))
          {
            ItemForce = setForce;
            if(setForce != 0)
            {
              Notify("The item will be pushed out with force: <color=yellow>"+setForce+"</color>");
            }
            else
            {
              Notify("Item will not be pushed out.");
            }
          }
          else
          {
            Notify("<color=red>You have not entered the number!</color>");
          }
          })
        }),
        new DialogButton("Cancel", true, (UnityAction)(() => dialog.Close())));
      }));
	  
      GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("snapAngleButton", "Snap angle", "Snap angle", () =>
      {
        SnapAngle = !SnapAngle;
        Notify(SnapAngle ? "The angle of the spawned object will be snapped to the spawner." : "The angle of the spawned object will not be snapped to the spawner.");
      }));
	  
      GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("freezeItemButton", "Freeze item", "Freeze item", () =>
      {
        FreezeItem = !FreezeItem;
        Notify(FreezeItem ? "The spawned item will be frozen." : "The spawned item will not be frozen.");
      }));
	  
      GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("itemActivationButton", "Item activation", "Item activation", () =>
      {
        StartUse = !StartUse;
        Notify(StartUse ? "The spawned item will be activated." : "The spawned item will not be activated.");
      }));
	  
      GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("resetButton", "Reset", "Reset", () =>
      {
        ResetSpawnerSettings();
        Notify("Settings reset successfully!");
      }));
	  
      GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("advancedModeButton", "Advanced mode", "Advanced mode", () =>
      {
        AdvancedMode = !AdvancedMode;
        Notify(AdvancedMode ? "<color=red>Advanced mode enabled! The advanced mode allows you to spawn any item from the game, it can ruin the gameplay and also some spawner functions will not work.</color>" : "Advanced mode is disabled.");
      }));
	  
      GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(
      new ContextMenuButton("endButton", "-----------", "-----------", () =>
      {
      }));
    }
    //Use
    private void Use()
    {
      if(!ParticleMode)
      {
        Quaternion angle = (SnapAngle ? transform.rotation : Quaternion.identity);
        if(!AdvancedMode)
        {
          SpawnableAsset AssetItem;
          GameObject SpawnedItem;
          if(ItemName != "Zombie")
          {
            AssetItem = ModAPI.FindSpawnable(ItemName);
            SpawnedItem = UnityEngine.Object.Instantiate(AssetItem.Prefab, transform.position, angle);
            SpawnedItem.transform.position += transform.up * LineHeight;
            SpawnedItem.name = AssetItem.name;
            CatalogBehaviour.PerformMod(AssetItem, SpawnedItem);
          }
          else
          {
            AssetItem = ModAPI.FindSpawnable("Human");
            SpawnedItem = UnityEngine.Object.Instantiate(AssetItem.Prefab, transform.position, angle);
            SpawnedItem.transform.position += transform.up * LineHeight;
            SpawnedItem.name = "Zombie";
            var person = SpawnedItem.GetComponent<PersonBehaviour>();
            for(int i = 0; i < person.Limbs.Length; i++)
            {
              person.Limbs[i].IsZombie = true;
              person.Limbs[i].SkinMaterialHandler.RottenProgress = 1f;
            }
          }
          ApplyChanges(SpawnedItem);
          if(ItemScale != 0)
          {
            SpawnedItem.transform.localScale *= ItemScale;
          }
          if(StartUse)
          {
            SpawnedItem.BroadcastMessage("Use");
          }
        }
        else
        {
          UnityEngine.Object[] prefabs = Resources.FindObjectsOfTypeAll(typeof(GameObject));
          foreach(UnityEngine.Object prefab in prefabs)
          {
            if(prefab.name == ItemName)
            {
              var obj = UnityEngine.Object.Instantiate(prefab, transform.position, angle) as GameObject;
              obj.transform.position += transform.up * LineHeight;
              ApplyChanges(obj);
              break;
            }
          }
        }
      }
      else
      {
        GameObject SpawnedParticle = ModAPI.CreateParticleEffect(ItemName, transform.position);
        SpawnedParticle.transform.position += transform.up * LineHeight;
        if(SnapAngle)
        {
          SpawnedParticle.transform.eulerAngles = transform.eulerAngles;
        }
      }
    }
	
    private void ApplyChanges(GameObject obj)
    {
      if(obj.transform.childCount > 0)
      {
        List<GameObject> childrenList = new List<GameObject>();
        Transform[] children = obj.transform.GetComponentsInChildren<Transform>(true);
        for (int i = 0; i < children.Length; i++)
        {
          Transform child = children[i];
          if (child != obj.transform)
          {
            childrenList.Add(child.gameObject);
          }
        }
        for (int i = 0; i < childrenList.Count; i++)
        {
          if(FreezeItem)
          {
            if(childrenList[i].GetComponent<PhysicalBehaviour>())
            {
              childrenList[i].AddComponent<FreezeBehaviour>();
            }
            if(obj.GetComponent<PhysicalBehaviour>())
            {
              obj.AddComponent<FreezeBehaviour>();
            }
          }
          if(ItemForce != 0)
          {
            if(childrenList[i].GetComponent<PhysicalBehaviour>())
            {
              childrenList[i].GetComponent<PhysicalBehaviour>().rigidbody.AddForce(transform.up * ItemForce, ForceMode2D.Impulse);
            }
            if(obj.GetComponent<PhysicalBehaviour>())
            {
              obj.GetComponent<PhysicalBehaviour>().rigidbody.AddForce(transform.up * ItemForce, ForceMode2D.Impulse);
            }
          }
          if(ItemCharge != 0)
          {
            if(childrenList[i].GetComponent<PhysicalBehaviour>())
            {
              childrenList[i].GetComponent<PhysicalBehaviour>().charge = ItemCharge;
            }
            if(obj.GetComponent<PhysicalBehaviour>())
            {
              obj.GetComponent<PhysicalBehaviour>().charge = ItemCharge;
            }
          }
		  if(ItemTemperature != 0)
          {
            if(childrenList[i].GetComponent<PhysicalBehaviour>())
            {
              childrenList[i].GetComponent<PhysicalBehaviour>().Temperature = ItemTemperature;
            }
            if(obj.GetComponent<PhysicalBehaviour>())
            {
              obj.GetComponent<PhysicalBehaviour>().Temperature = ItemTemperature;
            }
          }
        }
        if(ItemForce != 0)
        {
          for (int i = 0; i < obj.transform.childCount; i++)
          {
            PhysicalBehaviour phys;
            if (obj.transform.GetChild(i).TryGetComponent<PhysicalBehaviour>(out phys))
            {
              phys.rigidbody.AddForce(transform.up * ItemForce, ForceMode2D.Impulse);
            }
          }
        }
      }
      else
      {
        if(FreezeItem)
        {
          if(obj.GetComponent<PhysicalBehaviour>())
            obj.AddComponent<FreezeBehaviour>();
        }
        if(ItemForce != 0)
        {
          if(obj.GetComponent<PhysicalBehaviour>())
            obj.GetComponent<PhysicalBehaviour>().rigidbody.AddForce(transform.up * ItemForce, ForceMode2D.Impulse);
        }
        if(ItemCharge != 0)
        {
          if(obj.GetComponent<PhysicalBehaviour>())
            obj.GetComponent<PhysicalBehaviour>().charge = ItemCharge;
        }
        if(ItemTemperature != 0)
        {
          if(obj.GetComponent<PhysicalBehaviour>())
            obj.GetComponent<PhysicalBehaviour>().Temperature = ItemTemperature;
        }
      }
    }
	
    public void ResetSpawnerSettings()
    {
      ItemName = "Brick";
      LineHeight = 1.5f;
      ItemScale = 0f;
      ItemCharge = 0f;
      ItemForce = 0f;
	  ItemTemperature = 0f;
      FreezeItem = false;
      SnapAngle = false;
      StartUse = false;
      s_color = Color.green;
      SetSpawnerColor(s_color);
      SpawnerLine.SetPositions(new[] {Vector3.zero, Vector3.up * LineHeight});
    }
	
    public void SetSpawnerColor(Color color)
    {
      SpawnerGlow.GetComponent<SpriteRenderer>().color = color;
      SpawnerLine.startColor = color;
      SpawnerLine.endColor = new Color(0, 0, 0, 0);
    }
	
    private void Notify(string n)
    {
      ModAPI.Notify("<color=red>[</color><color=yellow>Notification</color><color=red>]</color> "+n);
    }
  }
}
